==> bee_hotel-2.13.x-dev.tar.gz: bee_hotel/src/Util/BeeHotelUnit.php <==
<?php

namespace Drupal\bee_hotel\Util;

use Drupal\commerce_product\Entity\Product;
use Drupal\node\Entity\Node;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Manage the BeeHotel Unit.
 */
class BeeHotelUnit {

  /**
   * Is this node a beehotelunit?
   *
   * @param Drupal\node\Entity\Node $node
   *   A node to be validated.
   *
   * @return bool
   *   Bool TRUE | FALSE
   */
  public function isThisNodeBeeHotel(Node $node) {
    $bee_settings = \Drupal::config('node.type.' . $node->bundle())->get('bee');
    if (isset($bee_settings)  &&  $bee_settings['bookable'] == 1) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * A List of Bee Hotel Units.
   *
   * @param array $options
   *   An array of options.
   *
   * @return array
   *   An array of nodes.
   */
  public function beeHotelUnits(array $options) {
    $nids = \Drupal::entityQuery('node')->condition('field_accept_reservations', 1)->execute();
    return Node::loadMultiple($nids);
  }

  /**
   * Is this node a beehotelunit?
   *
   * @param array $options
   *   Options query.
   *
   * @return bool
   *   Bool TRUE | FALSE
   */
  public function getBeeHotelUnits(array $options) {
    $query = \Drupal::entityQuery('node')->condition('status', 1)->condition('field_accept_reservations', 1, '=');
    $nids = $query->execute();
    return Node::loadMultiple($nids);
  }

  /**
   * Get a list of beeHotel Units receiving reservations.
   *
   * @return array
   *   Give a list of units.
   */
  public function getReceivingUnits() {
    $storage = \Drupal::entityTypeManager()->getStorage('bat_unit');
    $unit_ids = \Drupal::entityQuery('bat_unit')->execute();
    $bat_units = $storage->loadMultiple($unit_ids);
    $units = [];
    foreach ($bat_units as $bat_unit) {
      $bid = $bat_unit->Id();
      if (isset($bid)) {
        if ($unit = $this->getUnitFromBid($bid)) {
          if ($unit['node']->get("field_accept_reservations")->value == 1) {
            $units[$bid] = $unit;
          }
        }
      }
    }

    if (count($units) == 0) {
      $tmp = ("No Unit is receiving reservation...");
      \Drupal::messenger()->addError($tmp);
      (new RedirectResponse('/admin/content'))->send();
      exit();
    }

    return $units;
  }

  /**
   * Get Units avaialble in a given time range.
   *
   * @param array $data
   *   A structured array.
   *
   * @return array
   *   An array with useful data.
   */
  public function getAvailableUnits(array &$data) {

    $utilDates = new Dates($data);
    $utilDates->normaliseDatesFromSearchForm($data);
    $data['event_type'] = 'availability_daily';

    foreach (bat_unit_type_ids() as $k => $v) {
      $data['unit_types'][$k] = $k;
    }

    $data['available_units'] = bat_event_get_matching_units(
      $start_date = $data['norm']['dates_from_search_form']['checkin']['object']['day'],
      $end_date = $data['norm']['dates_from_search_form']['lastnight']['object'],
      $valid_name_states = ['bee_daily_available'],
      $types_id = $data['unit_types'],
      $event_type = $data['event_type'],
      $intersect = NULL,
      $drupal_units = NULL,
    );
    return $data;
  }

  /**
   * Load data for the BeeHotel unit.
   */
  public function getUnitFromBid($bid) {
    $unit = [];
    $unit['bid'] = $bid;
    $unit['node'] = $this->getUnitNode($unit);

    if ($unit['node'] === FALSE) {
      return;
    }

    $unit['pid'] = $unit['node']->get("field_product")[0]->target_id;
    $unit['product'] = Product::load((int) $unit['pid']);
    $unit['store'] = $this->getUnitStore($unit);
    $unit['bat'] = current(\Drupal::entityTypeManager()->getStorage('bat_unit')->loadByProperties(['id' => $bid]));
    $unit['bee_settings'] = \Drupal::config('node.type.' . $unit['node']->bundle())->get('bee');
    $unit['product_variations'] = $unit['product']->getVariations();
    $unit['images']['main'] = $this->getUnitMainImage($unit['node']);
    return $unit;
  }

  /**
   * For a given Unit, get the Store.
   */
  public function getUnitStore($unit) {
    $unit['tmp'] = $unit['product']->get("stores");
    $unit['tmp'] = $unit['tmp']->getValue();
    $store = \Drupal::entityTypeManager()
      ->getStorage('commerce_store')
      ->load(reset($unit['tmp'])['target_id']);
    return $store;
  }

  /**
   * Get the node of a BEE Hotel BAT unit.
   */
  public function getUnitNode($unit) {
    $node = \Drupal::entityTypeManager()
      ->getStorage('node')
      ->loadByProperties([
        'field_availability_daily' => $unit['bid'],
      ]);
    return reset($node);
  }

  /**
   * Get the node of a Bee Hotel Commerce product variation.
   */
  public function getVariationNode($variation) {
    $product = $variation->getProduct();
    $node = \Drupal::entityTypeManager()
      ->getStorage('node')
      ->loadByProperties([
        'field_product' => $product->Id(),
      ]);
    return reset($node);
  }

  /**
   * Get the main image for a given node.
   */
  public function getUnitMainImage(Node $node) {
    $data = [];
    $data['image'] = NULL;
    $status = "0";
    if ($status != 1) {
      return $data['image'];
    }
    // Field name field_image_large is a standard in the bee hotel enviroment.
    if (!empty($node->get("field_image_large")->target_id)) {
      $display_options = [
        'label' => 'hidden',
        'type' => 'entity_reference_entity_view',
        'settings' => [
          'view_mode' => 'some_media_image_view_mode',
        ],
      ];
      $data['image'] = $node->field_image_large->view($display_options);
    }
    else {
      $output = "This Unit node has no valid cover image, which is required";
      return $output;
    }
    return $data;
  }

  /**
   * Get current variation.
   */
  public function getCurrentVariation($unit, $data) {
    if (!empty($unit['node'])) {
      foreach ($unit['product_variations'] as $variation) {
        $data['request_guests'] = (int) $data['values']['guests'];
        $data['variation_max_occupancy'] = $this->maxOccupancy($unit['node']);
        if ($data['request_guests'] <= $data['variation_max_occupancy']) {
          return $variation;
        }
      }
    }
  }

  /**
   * Get the product for a given node.
   */
  public function getNodeProduct(Node $node) {
    if (isset($node)) {
      $data = [];
      $data['pid'] = $node->get("field_product")->target_id;
      $data['product'] = Product::load((int) $data['pid']);
      return $data['product'];
    }
  }

  /**
   * Get variations for a given product.
   */
  public function getProductVariations(Product $product) {
    if (isset($product)) {
      $data = [];
      $entity_manager = \Drupal::entityTypeManager();
      $data['vids'] = $product->getVariationIds();
      $options = [];
      foreach ($data['vids'] as $v) {
        $data['variation'][$v] = $entity_manager->getStorage('commerce_product_variation')->load((int) $v);
      }
      return $data['variation'];
    }
  }

  /**
   * Find max number occupancy across variants.
   */
  public function maxOccupancy($node) {

    $data = [];
    if (isset($node)) {
      $data['product'] = $this->getNodeProduct($node);
      $data['variations'] = $this->getProductVariations($data['product']);
      foreach ($data['variations'] as $vid => $variation) {
        $data['occupancy'][] = $variation->get('field_max_occupancy')->value;
      }
    }
    else {
      $units = $this->getReceivingUnits();
      foreach ($units as $unit) {
        foreach ($unit['product_variations'] as $variation) {
          $value = $variation->get("field_max_occupancy")->value;
          if (isset($value)) {
            $data['occupancy'][] = $value;
          }
        }
      }
    }
    if (count($data) == 0) {
      $tmp = ("Something wrong with commerce setup...");
      \Drupal::messenger()->addError($tmp);
      (new RedirectResponse('/admin/commerce'))->send();
      exit();
    }
    return (int) max($data['occupancy']);
  }

  /**
   * Store usefull data in session.
   *
   * @param array $data
   *
   *   Return array.
   */
  public function storeInSession(array $data) {
    $session = \Drupal::request()->getSession();
    $session->set('beehotel_data', $data);
  }

  /**
   * Get night for an given entity.
   */
  public function beeHotelCalculateNights(&$entity) {

    $tmp = [];
    $dateTime = new DrupalDateTime($entity->get('field_check_in')->value, 'UTC');
    $tmp['checkin_timestamp'] = $dateTime->getTimestamp();
    $dateTime = new DrupalDateTime($entity->get('field_check_out')->value, 'UTC');
    $tmp['checkout_timestamp'] = $dateTime->getTimestamp();
    $tmp['days'] = (int) ceil(($tmp['checkout_timestamp'] - $tmp['checkin_timestamp']) / 60 / 60 / 24);
    $entity->set('field_nights', $tmp['days']);

  }

  /**
   * Check the node price table is correctly filled.
   */
  public function validateBasePriceTable(EntityInterface $node) {

    $data = [];
    $data['config'] = \Drupal::config('beehotel_pricealterator.settings');
    $data['util'] = \Drupal::service('beehotel_pricealterator.util');
    $data['days'] = $data['util']->days();

    $status = 1;
    foreach ($data['days'] as $k => $label) {
      $tmp = [];
      $tmp['nid_day_high'] = $node->Id() . "_" . $k . "_high";
      $tmp['nid_day_low'] = $node->Id() . "_" . $k . "_low";
      foreach ($tmp as $neededvar) {
        if ($data['config']->get($neededvar) == NULL  ||  !is_numeric($data['config']->get($neededvar))) {
          $status = 0;
        }
      }
    }
    return $status;
  }

  /**
   * Enable reservations for Bee Hotel Units.
   */
  public function beeHotelUnitsEnableAcceptReservations() {

    $data = [];
    $data['config'] = \Drupal::config('beehotel.settings');

    foreach ($data['config']->getRawData() as $key => $value) {
      $variables[$key] = $value;
      if ($value <= time()) {
        $data['tmp'] = explode("_", $key);
        $data['node'] = Node::load($data['tmp'][2]);
        $data['node']->set('field_accept_reservations', 1);
        $data['node']->save();
      }
    }
  }

  /**
   * Store pause from reservation.
   *
   * When a BHU is paused from accepting reservation, store data in
   * module configuration.
   *
   * @param \Drupal\Core\Entity\EntityInterface $node
   *
   *   Current node.
   */
  public function registerAcceptReservationPause(EntityInterface $node) {
    $data = [];
    $data['config'] = \Drupal::service('config.factory')->getEditable('beehotel.settings');
    $timestamp = time() + ($node->get("field_accept_reservations")->value * 60 * 60);
    $data['config']->set("reservation_paused_" . $node->Id(), $timestamp)->save();
  }

}

==> bee_hotel-2.14.x-dev.tar.gz: bee_hotel/src/Util/BeeHotelUnit.php <==
<?php

namespace Drupal\bee_hotel\Util;

use Drupal\commerce_product\Entity\Product;
use Drupal\node\Entity\Node;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Manage the BeeHotel Unit.
 */
class BeeHotelUnit {

  /**
   * Is this node a beehotelunit?
   *
   * @param Drupal\node\Entity\Node $node
   *   A node to be validated.
   *
   * @return bool
   *   Bool TRUE | FALSE
   */
  public function isThisNodeBeeHotel(Node $node) {
    $bee_settings = \Drupal::config('node.type.' . $node->bundle())->get('bee');
    if (isset($bee_settings)  &&  $bee_settings['bookable'] == 1) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * A List of Bee Hotel Units.
   *
   * @param array $options
   *   An array of options.
   *
   * @return array
   *   An array of nodes.
   */
  public function beeHotelUnits(array $options) {
    $nids = \Drupal::entityQuery('node')->condition('field_accept_reservations', 1)->execute();
    return Node::loadMultiple($nids);
  }

  /**
   * Is this node a beehotelunit?
   *
   * @param array $options
   *   Options query.
   *
   * @return bool
   *   Bool TRUE | FALSE
   */
  public function getBeeHotelUnits(array $options) {
    $query = \Drupal::entityQuery('node')->condition('status', 1)->condition('field_accept_reservations', 1, '=');
    $nids = $query->execute();
    return Node::loadMultiple($nids);
  }

  /**
   * Get a list of beeHotel Units receiving reservations.
   *
   * @return array
   *   Give a list of units.
   */
  public function getReceivingUnits() {
    $storage = \Drupal::entityTypeManager()->getStorage('bat_unit');
    $unit_ids = \Drupal::entityQuery('bat_unit')->execute();
    $bat_units = $storage->loadMultiple($unit_ids);
    $units = [];
    foreach ($bat_units as $bat_unit) {
      $bid = $bat_unit->Id();
      if (isset($bid)) {
        if ($unit = $this->getUnitFromBid($bid)) {
          if ($unit['node']->get("field_accept_reservations")->value == 1) {
            $units[$bid] = $unit;
          }
        }
      }
    }

    if (count($units) == 0) {
      $tmp = ("No Unit is receiving reservation...");
      \Drupal::messenger()->addError($tmp);
      (new RedirectResponse('/admin/content'))->send();
      exit();
    }

    return $units;
  }

  /**
   * Get Units avaialble in a given time range.
   *
   * @param array $data
   *   A structured array.
   *
   * @return array
   *   An array with useful data.
   */
  public function getAvailableUnits(array &$data) {

    $utilDates = new Dates($data);
    $utilDates->normaliseDatesFromSearchForm($data);
    $data['event_type'] = 'availability_daily';

    foreach (bat_unit_type_ids() as $k => $v) {
      $data['unit_types'][$k] = $k;
    }

    $data['available_units'] = bat_event_get_matching_units(
      $start_date = $data['norm']['dates_from_search_form']['checkin']['object']['day'],
      $end_date = $data['norm']['dates_from_search_form']['lastnight']['object'],
      $valid_name_states = ['bee_daily_available'],
      $types_id = $data['unit_types'],
      $event_type = $data['event_type'],
      $intersect = NULL,
      $drupal_units = NULL,
    );
    return $data;
  }

  /**
   * Load data for the BeeHotel unit.
   */
  public function getUnitFromBid($bid) {
    $unit = [];
    $unit['bid'] = $bid;
    $unit['node'] = $this->getUnitNode($unit);

    if ($unit['node'] === FALSE) {
      return;
    }

    $unit['pid'] = $unit['node']->get("field_product")[0]->target_id;
    $unit['product'] = Product::load((int) $unit['pid']);
    $unit['store'] = $this->getUnitStore($unit);
    $unit['bat'] = current(\Drupal::entityTypeManager()->getStorage('bat_unit')->loadByProperties(['id' => $bid]));
    $unit['bee_settings'] = \Drupal::config('node.type.' . $unit['node']->bundle())->get('bee');
    $unit['product_variations'] = $unit['product']->getVariations();
    $unit['image_large'] = $this->getUnitMainImage($unit['node']);
    return $unit;
  }

  /**
   * For a given Unit, get the Store.
   */
  public function getUnitStore($unit) {
    $unit['tmp'] = $unit['product']->get("stores");
    $unit['tmp'] = $unit['tmp']->getValue();
    $store = \Drupal::entityTypeManager()
      ->getStorage('commerce_store')
      ->load(reset($unit['tmp'])['target_id']);
    return $store;
  }

  /**
   * Get the node of a BEE Hotel BAT unit.
   */
  public function getUnitNode($unit) {
    $node = \Drupal::entityTypeManager()
      ->getStorage('node')
      ->loadByProperties([
        'field_availability_daily' => $unit['bid'],
      ]);
    return reset($node);
  }

  /**
   * Get the node of a Bee Hotel Commerce product variation.
   */
  public function getVariationNode($variation) {
    $product = $variation->getProduct();
    $node = \Drupal::entityTypeManager()
      ->getStorage('node')
      ->loadByProperties([
        'field_product' => $product->Id(),
      ]);
    return reset($node);
  }

  /**
   * Get the main image for a given node.
   */
  public function getUnitMainImage(Node $node) {

    $image_large = [];
    $image_large['tid'] = $node->get("field_image_large")->target_id;

    // Field name field_image_large is a standard in the bee hotel enviroment.
    // @todo set style_name via UI
    if (!empty($image_large['tid'])) {
      $image_large['uri'] = isset($node->get('field_image_large')->entity) ? $node->get('field_image_large')->entity->getFileUri() : NULL;

      if ($image_large['uri']) {
        $image_large['image'] = [
          '#theme' => 'image_style',
          '#style_name' => 'image_large_teaser',
          '#uri' => $image_large['uri'],
        ];
        $image_large['markup'] = render($image_large['image']);
      }
    }
    else {
      $output = "This Unit node has no valid cover image, which is required";
      return $output;
    }
    return $image_large;
  }

  /**
   * Get current variation.
   */
  public function getCurrentVariation($unit, $data) {
    if (!empty($unit['node'])) {
      foreach ($unit['product_variations'] as $variation) {
        $data['request_guests'] = (int) $data['values']['guests'];
        $data['variation_max_occupancy'] = $this->maxOccupancy($unit['node']);
        if ($data['request_guests'] <= $data['variation_max_occupancy']) {
          return $variation;
        }
      }
    }
  }

  /**
   * Get the product for a given node.
   */
  public function getNodeProduct(Node $node) {
    if (isset($node)) {
      $data = [];
      $data['pid'] = $node->get("field_product")->target_id;
      $data['product'] = Product::load((int) $data['pid']);
      return $data['product'];
    }
  }

  /**
   * Get variations for a given product.
   */
  public function getProductVariations(Product $product) {
    if (isset($product)) {
      $data = [];
      $entity_manager = \Drupal::entityTypeManager();
      $data['vids'] = $product->getVariationIds();
      $options = [];
      foreach ($data['vids'] as $v) {
        $data['variation'][$v] = $entity_manager->getStorage('commerce_product_variation')->load((int) $v);
      }
      return $data['variation'];
    }
  }

  /**
   * Find max number occupancy across variants.
   */
  public function maxOccupancy($node) {

    $data = [];
    if (isset($node)) {
      $data['product'] = $this->getNodeProduct($node);
      $data['variations'] = $this->getProductVariations($data['product']);
      foreach ($data['variations'] as $vid => $variation) {
        $data['occupancy'][] = $variation->get('field_max_occupancy')->value;
      }
    }
    else {
      $units = $this->getReceivingUnits();
      foreach ($units as $unit) {
        foreach ($unit['product_variations'] as $variation) {
          $value = $variation->get("field_max_occupancy")->value;
          if (isset($value)) {
            $data['occupancy'][] = $value;
          }
        }
      }
    }
    if (count($data) == 0) {
      $tmp = ("Something wrong with commerce setup...");
      \Drupal::messenger()->addError($tmp);
      (new RedirectResponse('/admin/commerce'))->send();
      exit();
    }
    return (int) max($data['occupancy']);
  }

  /**
   * Store usefull data in session.
   *
   * @param array $data
   *
   *   Return array.
   */
  public function storeInSession(array $data) {
    $session = \Drupal::request()->getSession();
    $session->set('beehotel_data', $data);
  }

  /**
   * Get night for an given entity.
   */
  public function beeHotelCalculateNights(&$entity) {

    $tmp = [];
    $dateTime = new DrupalDateTime($entity->get('field_check_in')->value, 'UTC');
    $tmp['checkin_timestamp'] = $dateTime->getTimestamp();
    $dateTime = new DrupalDateTime($entity->get('field_check_out')->value, 'UTC');
    $tmp['checkout_timestamp'] = $dateTime->getTimestamp();
    $tmp['days'] = (int) ceil(($tmp['checkout_timestamp'] - $tmp['checkin_timestamp']) / 60 / 60 / 24);
    $entity->set('field_nights', $tmp['days']);

  }

  /**
   * Check the node price table is correctly filled.
   */
  public function validateBasePriceTable(EntityInterface $node) {

    $data = [];
    $data['config'] = \Drupal::config('beehotel_pricealterator.settings');
    $data['util'] = \Drupal::service('beehotel_pricealterator.util');
    $data['days'] = $data['util']->days();

    $status = 1;
    foreach ($data['days'] as $k => $label) {
      $tmp = [];
      $tmp['nid_day_high'] = $node->Id() . "_" . $k . "_high";
      $tmp['nid_day_low'] = $node->Id() . "_" . $k . "_low";
      foreach ($tmp as $neededvar) {
        if ($data['config']->get($neededvar) == NULL  ||  !is_numeric($data['config']->get($neededvar))) {
          $status = 0;
        }
      }
    }
    return $status;
  }

  /**
   * Enable reservations for Bee Hotel Units.
   */
  public function beeHotelUnitsEnableAcceptReservations() {

    $data = [];
    $data['config'] = \Drupal::config('beehotel.settings');

    foreach ($data['config']->getRawData() as $key => $value) {
      $variables[$key] = $value;
      if ($value <= time()) {
        $data['tmp'] = explode("_", $key);
        $data['node'] = Node::load($data['tmp'][2]);
        $data['node']->set('field_accept_reservations', 1);
        $data['node']->save();
      }
    }
  }

  /**
   * Store pause from reservation.
   *
   * When a BHU is paused from accepting reservation, store data in
   * module configuration.
   *
   * @param \Drupal\Core\Entity\EntityInterface $node
   *
   *   Current node.
   */
  public function registerAcceptReservationPause(EntityInterface $node) {
    $data = [];
    $data['config'] = \Drupal::service('config.factory')->getEditable('beehotel.settings');
    $timestamp = time() + ($node->get("field_accept_reservations")->value * 60 * 60);
    $data['config']->set("reservation_paused_" . $node->Id(), $timestamp)->save();
  }

}
